/************************************************************************/
/*										                        		*/
/*	main.c	-- Digilent RDK_RemoteControl reference design          	*/
/*											                        	*/
/************************************************************************/
/*	Author: 	Michael T. Alexander					            	*/
/*	Copyright 2009, Digilent Inc.					            		*/
/************************************************************************/
/*  Module Description: 						                		*/
/*											                        	*/
/*	This module demonstrates the use of the Digilent Remote Control		*/
/*	Robotic Development Kit (RDK-Remote) with the Cerbot 32MX4			*/
/*	and PmodJSTK.														*/
/*																		*/
/*	Note: Make sure that the Configuration Bits set in code option is	*/
/*	checked under Configure->Configuration Bits in MPLAB				*/
/*							                                            */
/************************************************************************/
/*  Revision History:						                			*/
/*											                        	*/
/*	04/28/2009 	(MichaelA): created			                			*/
/*	12/8/2010	(AaronO): renamed to RDK_RemoteControl					*/
/*											                        	*/
/************************************************************************/

/* ------------------------------------------------------------ */
/*				Include File Definitions	            	    */
/* ------------------------------------------------------------ */

#include <plib.h>
#include "stdtypes.h"
#include "util.h"
#include "spi2.h"
#include "config.h"
#include "MtrCtrl.h"

/* ------------------------------------------------------------ */
/*				Local Type Definitions			                */
/* ------------------------------------------------------------ */

#define		TCKPS22 			6
#define 	TCKPS21				5
#define 	TCKPS20				4

#define		TCKPS32 			6
#define 	TCKPS31				5
#define 	TCKPS30				4

/* ------------------------------------------------------------ */
/*				Global Variables				                */
/* ------------------------------------------------------------ */

#ifndef OVERRIDE_CONFIG_BITS

#pragma config ICESEL   = ICS_PGx2		// ICE/ICD Comm Channel Select
#pragma config BWP      = OFF			// Boot Flash Write Protect
#pragma config CP       = OFF			// Code Protect
#pragma config FNOSC    = PRIPLL		// Oscillator Selection
#pragma config FSOSCEN  = OFF			// Secondary Oscillator Enable
#pragma config IESO     = OFF			// Internal/External Switch-over
#pragma config POSCMOD  = HS			// Primary Oscillator
#pragma config OSCIOFNC = OFF			// CLKO Enable
#pragma config FPBDIV   = DIV_8			// Peripheral Clock divisor
#pragma config FCKSM    = CSDCMD		// Clock Switching & Fail Safe Clock Monitor
#pragma config WDTPS    = PS1			// Watchdog Timer Postscale
#pragma config FWDTEN   = OFF			// Watchdog Timer 
#pragma config FPLLIDIV = DIV_2			// PLL Input Divider
#pragma config FPLLMUL  = MUL_16		// PLL Multiplier
#pragma config UPLLIDIV = DIV_2			// USB PLL Input Divider
#pragma config UPLLEN   = OFF			// USB PLL Enabled
#pragma config FPLLODIV = DIV_1			// PLL Output Divider
#pragma config PWP      = OFF			// Program Flash Write Protect
#pragma config DEBUG    = OFF			// Debugger Enable/Disable
    
#endif


/* ------------------------------------------------------------ */
/*				Local Variables				                    */
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Forward Declarations			                */
/* ------------------------------------------------------------ */

void	DeviceInit( void );
void	AppInit( void );

/* ------------------------------------------------------------ */
/*				Interrupt Service Routines	            	    */
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Macros											*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Procedure Definitions			                */
/* ------------------------------------------------------------ */
/***	main
**
**	Synopsis:
**		st = main()
**
**	Parameters:
**		none
**
**	Return Values:
**		does not return
**
**	Errors:
**		none
**
**	Description:
**		Main program module. Performs basic board initialization
**		and then enters the main program loop.
*/

int main( void )
{
	HWORD posX;
	HWORD posY;
	BYTE btns;
	BOOL fLD2;
	char szTemp[20];
	
	fLD2 = fFalse;
	
	DeviceInit();
	AppInit();

	INTDisableInterrupts();
	
	while ( fTrue ) {

		INTDisableInterrupts();
		// Turn on LD1 of the PmodJSTK and get the position and button status.
		Spi2MaEnableSlave();				// enable the PmodJSTK
		DelayUs(15);
		if ( fLD2 ) {
			posX = Spi2MaTxByte(0x82);		// get lower byte of X position
			fLD2 = fFalse;
		}
		else {
			posX = Spi2MaTxByte(0x81);		// get lower byte of X position
			fLD2 = fTrue;
		}
		DelayUs(10);
		posX += ( Spi2MaTxByte(0) << 8 );	// get upper byte of X position
		DelayUs(10);
		posY = Spi2MaTxByte(0);				// get lower byte of Y position
		DelayUs(10);
		posY += ( Spi2MaTxByte(0) << 8 );	// get upper byte of Y position
		DelayUs(10);
		btns = Spi2MaTxByte(0);				// get button status
		DelayUs(15);
		Spi2MaDisableSlave();				// disable the PmodJSTK
		
		INTEnableInterrupts();

		//update motor direction and duty cycle
		UpdateMotorControl(posX, posY);
	}
}  // main()

/* ------------------------------------------------------------ */
/***	DeviceInit
**
**	Synopsis:
**		DeviceInit()
**
**	Parameters:
**		none
**
**	Return Values:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Initializes on chip peripheral devices to the default
**		state.
*/

void DeviceInit( void )
{
	// Configure UART1.
	U1BRG 		= 51;			// BAUD = 9600 bps when FPB = 8 MHz
	U1MODE		= ( 1 << 15 );	// enable UART1
	U1STASET 	= ( 1 << 10);	// enable transmitter
	
	// Configure SPI2.
	SPI2BRG = 7;	// Fsck = 500 KHz assuming Fpb = 8 MHz
	SPI2CON = ( 1 << SPI2_ON ) | ( 1 << SPI2_SMP ) | ( 1 << SPI2_CKE ) | ( 1 << SPI2_MSTEN );
	Spi2MaConfigSS();
	Spi2MaDisableSlave();
	
	// Configure LEDs.
	mPORTDSetPinsDigitalOut(BIT_0);
	mPORTDSetPinsDigitalOut(BIT_1);
	mPORTDSetPinsDigitalOut(BIT_2);
	mPORTDClearBits(BIT_0);
	mPORTDClearBits(BIT_1);
	mPORTDClearBits(BIT_2);
	
	// Configure IC1 for output.
	mPORTDSetPinsDigitalOut(BIT_8);
	mPORTDClearBits(BIT_8);
	
	// Configure IC2 for output.
	mPORTDSetPinsDigitalOut(BIT_9);
	mPORTDClearBits(BIT_9);

	/*Timer configuration for motors*/

	// Configure left motor direction pin and set default direction.
	trisMtrLeftDirClr	= ( 1 << bnMtrLeftDir );
	prtMtrLeftDirClr	= ( 1 << bnMtrLeftDir );	// forward
	
	// Configure right motor direction pin and set default direction.
	trisMtrRightDirClr	= ( 1 << bnMtrRightDir );	//modify for JD
	prtMtrRightDirSet	= ( 1 << bnMtrRightDir );	// forward

	// Configure Output Compare 2 to drive the left motor.
	OC2CON	= ( 1 << 2 ) | ( 1 << 1 );	// pwm
	OC2R	= dtcMtrStopped;
	OC2RS	= dtcMtrStopped;

	// Configure Output Compare 3.
	OC3CON = ( 1 << 3 ) | ( 1 << 2 ) | ( 1 << 1 );	// pwm
	OC3R	= dtcMtrStopped;
	OC3RS	= dtcMtrStopped;

	// Configure Timer 2.
	TMR2	= 0;									// clear timer 2 count
	PR2		= 9999;

	// Configure Timer 3.
	TMR3	= 0;
	PR3		= 9999;

	// Start timers and output compare units.
	T2CON		= ( 1 << 15 ) | ( 1 << TCKPS20 )|(1 << TCKPS21);		// timer 2 prescale = 8
	OC2CONSET	= ( 1 << 15 );	// enable output compare module 2
	OC3CONSET	= ( 1 << 15 );	// enable output compare module 3
	T3CON		= ( 1 << 15 ) | ( 1 << TCKPS31 ) | ( 1 << TCKPS30); //timer3 prescale = 8
	
	// Enable multi-vector interrupts.
	INTEnableSystemMultiVectoredInt();


}  // DeviceInit()

/* ------------------------------------------------------------ */
/***	AppInit
**
**	Synopsis:
**		AppInit()
**
**	Parameters:
**		none
**
**	Return Values:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Performs application specific initialization. Sets devices
**		and global variables for application.
*/

void AppInit( void )
{
}  // AppInit()

/*************************************************************************************/
